package net.knarcraft.blacksmith.command;

import net.knarcraft.blacksmith.BlacksmithPlugin;
import net.knarcraft.blacksmith.config.GlobalSetting;
import net.knarcraft.blacksmith.config.GlobalSettings;
import net.knarcraft.blacksmith.config.NPCSetting;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.enchantments.Enchantment;
import org.jetbrains.annotations.NotNull;

/**
 * The command used for changing global configuration options
 */
public class BlackSmithConfigCommand implements CommandExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                             @NotNull String[] args) {
        if (args.length > 0) {
            String commandName = args[0];
            if (commandName.equalsIgnoreCase("reload")) {
                return new ReloadCommand().onCommand(sender, command, label, args);
            }
            GlobalSettings settings = BlacksmithPlugin.getInstance().getSettings();

            //Changing reforge-able items' default isn't recommended
            if (commandName.equalsIgnoreCase(NPCSetting.REFORGE_ABLE_ITEMS.getCommandName())) {
                sender.sendMessage(ChatColor.DARK_RED + "Changing reforge-able items globally will make every new " +
                        "blacksmith unable to re-forge anything not in the list, unless it's changed for the " +
                        "individual NPC. If you really want to change this, change it manually.");
                return false;
            }
            
            if (args.length == 1) {
                //TODO: Display the current value of the setting
            } else if (args.length == 2 && isSpecialCase(commandName)) {
                //TODO: Display the current value for the specified setting and material
            }

            if (isSpecialCase(settings, commandName, args)) {
                return true;
            }

            for (GlobalSetting globalSetting : GlobalSetting.values()) {
                if (commandName.equalsIgnoreCase(globalSetting.getCommandName())) {
                    settings.changeValue(globalSetting, args[1]);
                    return true;
                }
            }
            for (NPCSetting npcSetting : NPCSetting.values()) {
                if (commandName.equalsIgnoreCase(npcSetting.getCommandName())) {
                    settings.changeValue(npcSetting, args[1]);
                }
            }
        }
        return false;
    }

    /**
     * Gets whether a command name matches a special case command
     * 
     * @param commandName <p>The command specified</p>
     * @return <p>True if the command is a special case</p>
     */
    private boolean isSpecialCase(String commandName) {
        return commandName.equalsIgnoreCase(GlobalSetting.BASE_PRICE.getCommandName()) || 
                commandName.equalsIgnoreCase(GlobalSetting.PRICE_PER_DURABILITY_POINT.getCommandName()) || 
                commandName.equalsIgnoreCase(GlobalSetting.ENCHANTMENT_COST.getCommandName());
    }

    /**
     * Gets whether the command could be processed as one of the three special cases
     *
     * @param settings    <p>The settings to modify</p>
     * @param commandName <p>The sub-command the player specified</p>
     * @param args        <p>All arguments given</p>
     * @return <p>True if already handled as a special case</p>
     */
    private boolean isSpecialCase(GlobalSettings settings, String commandName, String[] args) {
        if (commandName.equalsIgnoreCase(GlobalSetting.BASE_PRICE.getCommandName())) {
            settings.setBasePrice(Material.matchMaterial(args[1]), Double.parseDouble(args[2]));
            return true;
        }

        if (commandName.equalsIgnoreCase(GlobalSetting.PRICE_PER_DURABILITY_POINT.getCommandName())) {
            settings.setPricePerDurabilityPoint(Material.matchMaterial(args[1]), Double.parseDouble(args[2]));
            return true;
        }

        if (commandName.equalsIgnoreCase(GlobalSetting.ENCHANTMENT_COST.getCommandName())) {
            settings.setEnchantmentCost(Enchantment.getByKey(NamespacedKey.minecraft(args[1])), Double.parseDouble(args[2]));
            return true;
        }

        return false;
    }

}
