package net.knarcraft.blacksmith.command;

import net.citizensnpcs.api.CitizensAPI;
import net.citizensnpcs.api.npc.NPC;
import net.knarcraft.blacksmith.config.NPCSetting;
import net.knarcraft.blacksmith.trait.BlacksmithTrait;
import net.knarcraft.blacksmith.util.TypeValidationHelper;
import net.md_5.bungee.api.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.jetbrains.annotations.NotNull;

/**
 * The main command used for blacksmith editing
 */
public class BlackSmithEditCommand implements CommandExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                             @NotNull String[] args) {
        NPC npc = CitizensAPI.getDefaultNPCSelector().getSelected(sender);
        if (npc == null || !npc.hasTrait(BlacksmithTrait.class)) {
            sender.sendMessage(ChatColor.DARK_RED + "You must select an NPC before running this command");
            return true;
        }

        BlacksmithTrait blacksmithTrait = npc.getTrait(BlacksmithTrait.class);

        for (NPCSetting npcSetting : NPCSetting.values()) {
            String commandName = npcSetting.getCommandName();
            if (commandName.equalsIgnoreCase(args[0])) {
                String newValue = args.length < 2 ? null : args[1];
                return changeNPCSetting(blacksmithTrait, npcSetting, newValue, sender);
            }
        }
        return false;
    }

    /**
     * Changes the given NPC setting, or displays the current value if a new value isn't specified
     *
     * @param blacksmithTrait <p>The blacksmith trait belonging to the selected NPC</p>
     * @param npcSetting      <p>The NPC setting to change</p>
     * @param newValue        <p>The value to change the setting to</p>
     * @param sender          <p>The command sender to notify about results</p>
     * @return <p>True if everything went successfully</p>
     */
    private boolean changeNPCSetting(BlacksmithTrait blacksmithTrait, NPCSetting npcSetting, String newValue,
                                     CommandSender sender) {
        if (newValue == null) {
            //Display the current value of the setting
            sender.sendMessage(ChatColor.GREEN + "Current value of " + npcSetting.getCommandName() + ": " +
                    ChatColor.GOLD + blacksmithTrait.getSettings().getRawValue(npcSetting));
            return true;
        } else {
            boolean isValidType = TypeValidationHelper.isValid(npcSetting.getValueType(), newValue, sender);
            if (isValidType) {
                //Change the setting
                blacksmithTrait.getSettings().changeSetting(npcSetting,
                        ChatColor.translateAlternateColorCodes('&', newValue));
                sender.sendMessage(ChatColor.GREEN + npcSetting.getNodeName() + " set to " + ChatColor.GOLD + newValue);
                return true;
            } else {
                return false;
            }
        }
    }

}
